// Copyright (C) 2000-2007, Luca Padovani <padovani@sti.uniurb.it>.
// 
// This file is part of GtkMathView, a flexible, high-quality rendering
// engine for MathML documents.
// 
// GtkMathView is free software; you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
// 
// GtkMathView is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef __TemplateElementValidator_hh__
#define __TemplateElementValidator_hh__

#include "gmv_defines.h"

template <class Model>
class GMV_MathView_EXPORT TemplateElementValidator
{
public:
  TemplateElementValidator(const String& ns = "*", const String& n = "*")
    : namespaceURI(ns), name(n)
  { }

  bool valid(const typename Model::Node& p) const
  {
    assert(p);
    return (Model::getNodeType(p) == Model::ELEMENT_NODE)
      && (namespaceURI == "*" || namespaceURI == Model::getNodeNamespaceURI(p))
      && (name == "*" || name == Model::getNodeName(p));
  }

private:
  String namespaceURI;
  String name;
};

#endif // __TemplateElementValidator_hh__
